;;; mb-pos.el --- Indicate minibuffer-history-position in prompt
;;
;; Copyright (C) 2007 Free Software Foundation, Inc.
;;
;; Author: Juri Linkov <juri@jurta.org>
;; Keywords: convenience

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
;; Boston, MA 02110-1301, USA.

;;; Commentary:
;;
;; Defines the minor mode `minibuffer-history-position-indicate-mode'.
;;
;; When active, navigating in the minibuffer history with M-p or M-n keys
;; will show the current history position in the minibuffer prompt.

;;; Code:

(defvar minibuffer-history-position-indicator-function nil
  "If non-nil, function to set up the minibuffer history position indicator.
It is called with one argument, the minibuffer history position,
and must return a string.")

;; An overlay covering the prompt.  This is a buffer-local variable in
;; each affected minibuffer.
;;
(defvar minibuffer-history-position-overlay)
(make-variable-buffer-local 'minibuffer-history-position-overlay)

;; This function goes on goto-history-element-post-hook
(defun minibuffer-history-position-setup ()
  "Set up a minibuffer for `minibuffer-history-position-indicate-mode'.
The prompt should already have been inserted."
  (unless (overlayp minibuffer-history-position-overlay)
    (setq minibuffer-history-position-overlay
          (make-overlay (point-min) (1+ (point-min)))))
  (overlay-put minibuffer-history-position-overlay 'before-string
               (if minibuffer-history-position-indicator-function
                   (funcall minibuffer-history-position-indicator-function
                            minibuffer-history-position)
                 (propertize (format "%d:" minibuffer-history-position)
                             'face 'minibuffer-prompt)))
  (overlay-put minibuffer-history-position-overlay 'evaporate t))

;;;###autoload
(define-minor-mode minibuffer-history-position-indicate-mode
  "Toggle Minibuffer History Position Indication mode.
When active, navigating in the minibuffer history with M-p or M-n keys
will show the current history position in the minibuffer prompt.

With prefix argument ARG, turn on if positive, otherwise off.
Returns non-nil if the new state is enabled."
  :global t
  :group 'minibuffer
  (if minibuffer-history-position-indicate-mode
      (progn
        ;; Enable the mode
        (add-hook 'minibuffer-setup-hook 'minibuffer-history-position-setup)
        (add-hook 'goto-history-element-post-hook 'minibuffer-history-position-setup))
    ;; Disable the mode
    (remove-hook 'minibuffer-setup-hook 'minibuffer-history-position-setup)
    (remove-hook 'goto-history-element-post-hook 'minibuffer-history-position-setup)))

(provide 'mb-pos)

;; arch-tag: 50224089-5bf5-46f8-803d-18f018c5eacf
;;; mb-pos.el ends here
